@@ -1,27 +1,75 @@
 import jwt
 
-from django.utils.encoding import smart_text
+from django.contrib.auth import get_user_model
+from django.utils.encoding import smart_str
+from django.utils.translation import gettext_lazy as _
 from rest_framework import exceptions
-from rest_framework_jwt.settings import api_settings
-from rest_framework.authentication import (BaseAuthentication,
-                                           get_authorization_header)
+from rest_framework.authentication import (
+    BaseAuthentication, get_authorization_header
+)
 
-try:
-    from django.contrib.auth import get_user_model
-except ImportError:  # Django < 1.5
-    from django.contrib.auth.models import User
-else:
-    User = get_user_model()
+from rest_framework_jwt.settings import api_settings
 
 
 jwt_decode_handler = api_settings.JWT_DECODE_HANDLER
-jwt_get_user_id_from_payload = api_settings.JWT_PAYLOAD_GET_USER_ID_HANDLER
+jwt_get_username_from_payload = api_settings.JWT_PAYLOAD_GET_USERNAME_HANDLER
 
 
-class JSONWebTokenAuthentication(BaseAuthentication):
+class BaseJSONWebTokenAuthentication(BaseAuthentication):
     """
     Token based authentication using the JSON Web Token standard.
+    """
 
+    def authenticate(self, request):
+        """
+        Returns a two-tuple of `User` and token if a valid signature has been
+        supplied using JWT-based authentication.  Otherwise returns `None`.
+        """
+        jwt_value = self.get_jwt_value(request)
+        if jwt_value is None:
+            return None
+
+        try:
+            payload = jwt_decode_handler(jwt_value)
+        except jwt.ExpiredSignature:
+            msg = _('Signature has expired.')
+            raise exceptions.AuthenticationFailed(msg)
+        except jwt.DecodeError:
+            msg = _('Error decoding signature.')
+            raise exceptions.AuthenticationFailed(msg)
+        except jwt.InvalidTokenError:
+            raise exceptions.AuthenticationFailed()
+
+        user = self.authenticate_credentials(payload)
+
+        return (user, payload)
+
+    def authenticate_credentials(self, payload):
+        """
+        Returns an active user that matches the payload's user id and email.
+        """
+        User = get_user_model()
+        username = jwt_get_username_from_payload(payload)
+
+        if not username:
+            msg = _('Invalid payload.')
+            raise exceptions.AuthenticationFailed(msg)
+
+        try:
+            user = User.objects.get_by_natural_key(username)
+        except User.DoesNotExist:
+            msg = _('Invalid signature.')
+            raise exceptions.AuthenticationFailed(msg)
+
+        if not user.is_active:
+            msg = _('User account is disabled.')
+            raise exceptions.AuthenticationFailed(msg)
+
+        return user
+
+
+class JSONWebTokenAuthentication(BaseJSONWebTokenAuthentication):
+    """
     Clients should authenticate by passing the token key in the "Authorization"
     HTTP header, prepended with the string specified in the setting
     `JWT_AUTH_HEADER_PREFIX`. For example:
@@ -30,55 +78,27 @@ class JSONWebTokenAuthentication(BaseAuthentication):
     """
     www_authenticate_realm = 'api'
 
-    def authenticate(self, request):
-        """
-        Returns a two-tuple of `User` and token if a valid signature has been
-        supplied using JWT-based authentication.  Otherwise returns `None`.
-        """
+    def get_jwt_value(self, request):
         auth = get_authorization_header(request).split()
         auth_header_prefix = api_settings.JWT_AUTH_HEADER_PREFIX.lower()
 
-        if not auth or smart_text(auth[0].lower()) != auth_header_prefix:
+        if not auth:
+            if api_settings.JWT_AUTH_COOKIE:
+                return request.COOKIES.get(api_settings.JWT_AUTH_COOKIE)
+            return None
+
+        if smart_str(auth[0].lower()) != auth_header_prefix:
             return None
 
         if len(auth) == 1:
-            msg = 'Invalid Authorization header. No credentials provided.'
+            msg = _('Invalid Authorization header. No credentials provided.')
             raise exceptions.AuthenticationFailed(msg)
         elif len(auth) > 2:
-            msg = ('Invalid Authorization header. Credentials string '
-                   'should not contain spaces.')
+            msg = _('Invalid Authorization header. Credentials string '
+                    'should not contain spaces.')
             raise exceptions.AuthenticationFailed(msg)
 
-        try:
-            payload = jwt_decode_handler(auth[1])
-        except jwt.ExpiredSignature:
-            msg = 'Signature has expired.'
-            raise exceptions.AuthenticationFailed(msg)
-        except jwt.DecodeError:
-            msg = 'Error decoding signature.'
-            raise exceptions.AuthenticationFailed(msg)
-
-        user = self.authenticate_credentials(payload)
-
-        return (user, auth[1])
-
-    def authenticate_credentials(self, payload):
-        """
-        Returns an active user that matches the payload's user id and email.
-        """
-        try:
-            user_id = jwt_get_user_id_from_payload(payload)
-
-            if user_id is not None:
-                user = User.objects.get(pk=user_id, is_active=True)
-            else:
-                msg = 'Invalid payload'
-                raise exceptions.AuthenticationFailed(msg)
-        except User.DoesNotExist:
-            msg = 'Invalid signature'
-            raise exceptions.AuthenticationFailed(msg)
-
-        return user
+        return auth[1]
 
     def authenticate_header(self, request):
         """
@@ -86,4 +106,4 @@ class JSONWebTokenAuthentication(BaseAuthentication):
         header in a `401 Unauthenticated` response, or `None` if the
         authentication scheme should return `403 Permission Denied` responses.
         """
-        return 'JWT realm="{0}"'.format(self.www_authenticate_realm)
+        return '{0} realm="{1}"'.format(api_settings.JWT_AUTH_HEADER_PREFIX, self.www_authenticate_realm)
